// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; indent-tabs-mode: nil; -*-
////////////////////////////////////////////////////////////////////////
// File: timedgate2.h
// This file contains sample implementation for the test generation:
// Train gate controller with delayed stop implementation under test.
// Authors:
//   Marius Mikucionis marius@cs.aau.dk
////////////////////////////////////////////////////////////////////////

#ifndef TIMEDGATE_H
#define TIMEDGATE_H

#include "tron/adapter.h"
#include "tron/Executable.h"
#include "tron/timing.h"

#include <pthread.h>
#include <semaphore.h>
#include <queue>


class TronLock
{
    pthread_mutex_t &mutex;
public:
    TronLock(pthread_mutex_t& m): mutex(m) { tron_mutex_lock(&mutex); }
    ~TronLock() { tron_mutex_unlock(&mutex); }
};

class GateAdapter: public TestAdapter, protected Executable
{
public:
    GateAdapter(Reporter* rep, int railways, int mutant=0);
    virtual ~GateAdapter();
/* TestAdapter part: */
    void start_imp();
    void perform_imp(int32_t chan, uint16_t n, const int32_t data[]);

/* The gate controller thread part: */
    virtual void* threadExecute();

/* methods for reporting outputs: */
    void reportGo(const int &train);
    void reportStop(const int &train);

protected:
    volatile bool alive;
    volatile bool abort;
    int gatecnt;
    std::queue<int32_t> inpChan; // input buffer for channel
    std::queue<int32_t> inpTrain; // input buffer for train id
    
    pthread_mutex_t input_m;
    pthread_cond_t input_c, ack_c;
};

#endif
